package TestApp::I18N::de;

use strict;
use warnings;

use base 'TestApp::I18N';

our %Lexicon = (
  'English' => 'Englisch',
  'German'  => 'Deutsch',
  'French'  => 'Franzäsisch',
  'Italian' => 'Italienisch',
);

1;
